/**
 * File:   cWeapon.h
 * Author: hackCraft
 *
 * Created on March 5, 2008, 4:09 PM
 */

#ifndef _CWEAPON_H
#define	_CWEAPON_H

#include <list>
using namespace std;

class cWorld;
class cObject;


class cWeapon {
public:
    // Object whielding the weapon (normaly not damaged by weapon).
    cObject* mOwner;
    
    // Points To Current Vehicle Base Matrix.
    float* mBasefv;
    
    // Mounting Base Matrix relative to Vehicle Base Matrix.
    float* mMountfv;
    
    // Remaining reload time until next firing is possible.
    float mReloading;
    
public:
    cWeapon();
    cWeapon(cWeapon *source);

    // "Eignungsbeurteilung"
    //
    // Fuzzy-Value [0,1] indicating usability or fittness
    // of this weapon to be fired right now (not considering weapon state).
    // May sum up (depends on Weapon) (future):
    // +Ability/Possibility to hit target (may roll some dice).
    // +Possible damage on target.
    // +Possible damage on other enemy units (owner's enemies).
    // -Possible self-damage.
    // -Possible ally-damage.
    // +In (optimal) Firerange.
    // -Out of (optimal) Firerange.
    virtual float fit(cObject* target) { return 0.5f; };
    
    // Considering weapon state, true when weapon is
    // at hand, ready-loaded and unlocked.
    virtual bool ready() { return !(mReloading > +0.0f); };
    
    // Fire Weapon at target.
    virtual void fire(cObject* target) = 0;

    // Animation, transformation and rendering just like in cObject.
    // But called by cObject not by cWorld.
    virtual void animate(float spf) {};
    virtual void transform() {};
    virtual void drawSolid() {};
    virtual void drawEffect() {};
    virtual void drawHUD() {};
    
    // Utility to damage along a cylinder from the current position
    // of a projectile to the last position.
    // Returns amount of damaged objects.
    int damageAllByCylinder(float* position, float* velocity, float spf, float radius, int roles, float damage);
};


// The Machine-Gun is an (almost) instant Damager
// so Damage is inflicted to the target (almost)
// at the moment of pulling the trigger.
// What you can see when you are pulling the trigger
// are't the bullets but the flare ammunition
// every nth round of fire.
// Flare ammunition is used to coordinate Fire in
// an all-in attack - while in a stealth attack it
// may and will indicate your location to the enemy.
class cMachineGun : public cWeapon {
protected:
    struct Shrapnel {
        float pos[3];
        float vel[3];
        float cnt;
    };
    list<Shrapnel*> shrapnels;
    list<Shrapnel*> castoff;
    
public:
    cMachineGun();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid() {};
    virtual void drawEffect();
    virtual void drawHUD();
};


// The Plasma-Gun uses an ionized mixture
// of inert-gas and metal-slags which
// are vapourized and then accelerated by
// means of ultra high current.
// Therefore it is the ultra high power
// cousin of the ordinary railgun which
// is known for accelerating a metal
// particle between two rails.
// Problem with the older cousin has always
// been that when the current was strong
// enough to almost melt the particle then
// the particle would stick to the rails.
// In this more modern version inert-gas (as
// known from Arc-Welding) is used to prohibit
// redox-reactions (ie. burning like Thermit)
// while the slug is still in the Barrel.
// Therefore the Plasma-Gun is more like
// a high power mix of an old
// Television-Cathode-Ray-Tube (CRT) and a
// MIG-Welder with a vapourizing rail.
class cPlasmaGun : public cWeapon {
protected:
    struct Shrapnel {
        float pos[3];
        float vel[3];
        float cnt;
    };
    list<Shrapnel*> shrapnels;
    
public:
    cPlasmaGun();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid() {};
    virtual void drawEffect();
    virtual void drawHUD();
};


// The flaming Spark-Gun can be used as
// a means of demolition to destroy
// buildings and other nonmoving things faster.
class cSparkGun : public cWeapon {
protected:
    struct Shrapnel {
        float pos[3];
        float vel[3];
        float cnt;
    };
    list<Shrapnel*> shrapnels;
    
public:
    cSparkGun();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid() {};
    virtual void drawEffect();
};


// The Raybeam is an pulse charged beam
// of destroying rays intersecting (almost) everything.
// Prior to firing the Raybeam needs to
// charge its enourmous capacitor-banks.
// When finally firing the Raybeam it
// will take some time to pulse the beam
// to its full extends afterwards it will
// perish as the energy of the caps was
// fully drained from the banks.
class cRaybeam : public cWeapon {
protected:
    float raylife;

public:
    cRaybeam();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid();
    virtual void drawEffect();
    virtual void drawHUD();
};


// The Homing-Missile is in itself a firce
// enemy to be confronted against.
// Its brains has evolved from simple
// transistor-logik over fuzzy-logik to
// a full rebuild of the abilities of
// swarming insects and some even show
// some self-awareness.
// It is rumored that some lost or even
// deserted formed their own hives.
// That's what happens to lost missiles.
class cHomingMissile : public cWeapon {
protected:
    struct Smoke {
        float pos[3];
        float cnt;
        float timer;
    };
    struct Missile {
        float pos[3];
        float vel[3];
        float fuel; // remaining fuel in seconds
        float timer; // time since start
        float nextsmoke; // Countdown til spawning next Smoke-Particle.
        cObject* target;
        list<Smoke*> trail;
    };
    list<Missile*> missiles;
    
public:
    cHomingMissile();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid();
    virtual void drawEffect();
};


// Be aware of explosions - be they by exploding targets,
// by grenades or missiles. Explosions can cause severe
// damage in any random direction around the source of
// the detonation through shrapnel particles, concrete
// and ripped of machine parts.
class cExplosion : public cWeapon {
protected:
    struct Smoke {
        float pos[3];
        float cnt;
        float timer;
    };
    struct Missile {
        float pos[3];
        float vel[3];
        float fuel; // remaining fuel in seconds
        float timer; // time since start
        float nextsmoke; // Countdown til spawning next Smoke-Particle.
        cObject* target;
        list<Smoke*> trail;
    };
    list<Missile*> missiles;
    
public:
    cExplosion();
    
    virtual void fire(cObject *target);
    virtual void animate(float spf);
    virtual void drawSolid();
    virtual void drawEffect();
};


#endif	/* _CWEAPON_H */

